--- Initialises the database if it is not already initialised.
local function tally_ensure_database()
	if tally_npcs == nil then
		tally_npcs = {}
	end
end

--- Finds the existing or creates a new NPC database entry for npcId. Internal to TallyDatabase.
-- @param npcId Integer. The ID of the type of NPC to find or create.
local function tally_find_npc(npcId)
	tally_ensure_database()
	local entry = tally_npcs[npcId]
	if entry == nil then
		entry =
		{
			name = nil,
			kills = 0,
			loots = 0,
			locations = {},
			loot = {}
		}
		tally_npcs[npcId] = entry
	end
	return entry
end

--- Notes details of a mob that has presumably just been observed.
-- @param npcId Integer. The ID of the type of NPC that has been looted.
-- @param name String. The mob's (localised) name. This will be the display name for mobs with this ID in future.
-- @param location String. The zone name and subzone name where the mob should be noted as spawning.
function tally_record_mob(npcId, name, location)
	local entry = tally_find_npc(npcId)
	entry.name = name
	local locations = entry.locations
	if locations[location] == nil then
		locations[location] = 1
	end
end

--- Notes that a mob of type npcId has just been killed.
-- @param npcId Integer. The ID of the type of NPC that has been looted.
function tally_record_mob_kill(npcId)
	local entry = tally_find_npc(npcId)
	entry.kills = entry.kills + 1
end

--- Notes that a mob of type npcId has just been looted and has dropped items.
-- @param npcId Integer. The ID of the type of NPC that has been looted.
-- @param items Table. A table of itemId -> quantity.
function tally_record_mob_loot(npcId, items)
	local entry = tally_find_npc(npcId)
	entry.loots = entry.loots + 1
	local loot = entry.loot
	for itemId, quantity in pairs(items) do
		if loot[itemId] == nil then
			loot[itemId] = quantity
		else
			loot[itemId] = loot[itemId] + quantity
		end
	end
end

--- Notes that a mob of type npcId has just been observed running away in fear.
-- @param npcId Integer. The ID of the type of NPC that runs.
function tally_record_mob_runs(npcId)
	local entry = tally_find_npc(npcId)
	entry.runs = 1
end

